"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEvmNativeSigner = exports.EvmNativeSigner = exports.getEvmSignerForSigner = exports.getEvmSignerForKey = exports.getEvmSigner = void 0;
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
const ethers_1 = require("ethers");
const platform_js_1 = require("./platform.js");
const types_js_1 = require("./types.js");
const ethers_2 = require("ethers");
async function getEvmSigner(signer, opts, chain) {
    if (!isEthersSigner(signer)) {
        const { key, rpc } = signer;
        signer = new ethers_1.Wallet(key, rpc);
    }
    if (!chain) {
        const [, c] = await platform_js_1.EvmPlatform.chainFromRpc(signer.provider);
        chain = c;
    }
    signer = new ethers_2.NonceManager(signer);
    return new EvmNativeSigner(chain, await signer.getAddress(), signer, opts);
}
exports.getEvmSigner = getEvmSigner;
// Get a SignOnlySigner for the EVM platform
async function getEvmSignerForKey(rpc, privateKey) {
    return getEvmSigner({ rpc, key: privateKey });
}
exports.getEvmSignerForKey = getEvmSignerForKey;
// Get a SignOnlySigner for the EVM platform
async function getEvmSignerForSigner(chain, signer) {
    return getEvmSigner(signer, {}, chain);
}
exports.getEvmSignerForSigner = getEvmSignerForSigner;
class EvmNativeSigner extends sdk_connect_1.PlatformNativeSigner {
    opts;
    constructor(_chain, _address, _signer, opts) {
        super(_chain, _address, _signer);
        this.opts = opts;
    }
    chain() {
        return this._chain;
    }
    address() {
        return this._address;
    }
    async sign(tx) {
        const signed = [];
        // TODO: Better gas estimation/limits
        let maxFeePerGas = 1500000000n; // 1.5gwei
        let maxPriorityFeePerGas = 100000000n; // 0.1gwei
        // Celo does not support this call
        if (this.chain() !== 'Celo') {
            const feeData = await this._signer.provider.getFeeData();
            maxFeePerGas = feeData.maxFeePerGas ?? maxFeePerGas;
            maxPriorityFeePerGas =
                feeData.maxPriorityFeePerGas ?? maxPriorityFeePerGas;
        }
        let nonce = await this._signer.getNonce();
        for (const txn of tx) {
            const { transaction, description } = txn;
            console.log(`Signing: ${description} for ${this.address()}`);
            const t = {
                ...transaction,
                ...{
                    maxFeePerGas,
                    maxPriorityFeePerGas,
                    nonce,
                },
            };
            const estimate = await this._signer.provider.estimateGas(t);
            t.gasLimit = estimate + estimate / 10n; // Add 10% buffer
            if (this.opts?.maxGasLimit && t.gasLimit > this.opts?.maxGasLimit) {
                throw new Error(`Gas limit ${t.gasLimit} exceeds maxGasLimit ${this.opts?.maxGasLimit}`);
            }
            signed.push(await this._signer.signTransaction(t));
            // assuming no nonce manager
            nonce += 1;
        }
        return signed;
    }
}
exports.EvmNativeSigner = EvmNativeSigner;
function isEvmNativeSigner(signer) {
    return ((0, sdk_connect_1.isNativeSigner)(signer) &&
        (0, sdk_connect_1.chainToPlatform)(signer.chain()) === types_js_1._platform &&
        isEthersSigner(signer.unwrap()));
}
exports.isEvmNativeSigner = isEvmNativeSigner;
// No type guard provided by ethers, instanceof checks will fail on even slightly different versions of ethers
function isEthersSigner(thing) {
    return ('provider' in thing &&
        typeof thing.connect === 'function' &&
        typeof thing.getAddress === 'function' &&
        typeof thing.getNonce === 'function' &&
        typeof thing.populateCall === 'function' &&
        typeof thing.populateTransaction === 'function' &&
        typeof thing.estimateGas === 'function' &&
        typeof thing.call === 'function' &&
        typeof thing.resolveName === 'function' &&
        typeof thing.signTransaction === 'function' &&
        typeof thing.sendTransaction === 'function' &&
        typeof thing.signMessage === 'function' &&
        typeof thing.signTypedData === 'function');
}
//# sourceMappingURL=signer.js.map